/* This file is part of turtle, a turtle-graphics system for Scheme.

Copyright (C) 2008  Luca Saiu

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */


#include "debug.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdarg.h>

void turtle_fatal(const char *error_message_template, ...){
  /* Print the message using vfprintf(): */
  va_list variadic_parameters;
  va_start(variadic_parameters, error_message_template);
  printf("FATAL error: ");
  vfprintf(stdout, error_message_template, variadic_parameters);
  printf("\n");
  va_end(variadic_parameters);
  
  /* Exit with failure status code: */
  exit(EXIT_FAILURE);
}

void* turtle_malloc(const size_t size_in_bytes){
  void *result = malloc(size_in_bytes);
  if(result == NULL)
    turtle_fatal("malloc() failed when making a %i-byte buffer",
                 (int)size_in_bytes);
  else
    return result;
}

void* turtle_realloc(void *buffer, const size_t size_in_bytes){
  void *result = realloc(buffer, size_in_bytes);
  if(result == NULL)
    turtle_fatal("realloc() failed when resizing %p to %i bytes",
                 buffer,
                 (int)size_in_bytes);
  else
    return result;
}

void turtle_free(void *buffer){
  free(buffer);
}
